import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/bloc/device_mgmt_bloc/device_managment_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routiens/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routiens/widgets/dragable_card.dart';

class RoutineDevices extends StatelessWidget {
  const RoutineDevices({super.key});

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (context) => DeviceManagementBloc()..add(FetchDevices()),
      child: BlocBuilder<DeviceManagementBloc, DeviceManagementState>(
        builder: (context, state) {
          if (state is DeviceManagementLoaded) {
            List<AllDevicesModel> deviceList = state.devices
                .where((device) =>
                    device.productType == 'AC' ||
                    device.productType == '1G' ||
                    device.productType == '2G' ||
                    device.productType == '3G')
                .toList();

            // Provide the RoutineBloc to the child widgets
            return BlocBuilder<RoutineBloc, RoutineState>(
              builder: (context, routineState) {
                return Wrap(
                  spacing: 10,
                  runSpacing: 10,
                  children: deviceList.asMap().entries.map((entry) {
                    final device = entry.value;
                    if (routineState.searchText != null && routineState.searchText!.isNotEmpty) {
                      return device.name!
                              .toLowerCase()
                              .contains(routineState.searchText!.toLowerCase())
                          ? DraggableCard(
                              imagePath: device.getDefaultIcon(device.productType),
                              title: device.name ?? '',
                              deviceData: {
                                'device': device,
                                'imagePath': device.getDefaultIcon(device.productType),
                                'title': device.name ?? '',
                                'deviceId': device.uuid,
                                'productType': device.productType,
                                'functions': device.functions,
                                'uniqueCustomId': '',
                              },
                            )
                          : Container();
                    } else {
                      return DraggableCard(
                        imagePath: device.getDefaultIcon(device.productType),
                        title: device.name ?? '',
                        deviceData: {
                          'device': device,
                          'imagePath': device.getDefaultIcon(device.productType),
                          'title': device.name ?? '',
                          'deviceId': device.uuid,
                          'productType': device.productType,
                          'functions': device.functions,
                          'uniqueCustomId': '',
                        },
                      );
                    }
                  }).toList(),
                );
              },
            );
          }
          return const Center(child: CircularProgressIndicator());
        },
      ),
    );
  }
}
